#include <mega8.h>
#include <delay.h>
#include <alcd.h>
#include <stdio.h>

#define RELAY_1 PORTD.6
#define RELAY_2 PORTD.7


#define ADC_VREF_TYPE 0x60


#ifndef RXB8
#define RXB8 1
#endif

#ifndef TXB8
#define TXB8 0
#endif

#ifndef UPE
#define UPE 2
#endif

#ifndef DOR
#define DOR 3
#endif

#ifndef FE
#define FE 4
#endif

#ifndef UDRE
#define UDRE 5
#endif

#ifndef RXC
#define RXC 7
#endif

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<DOR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)

// USART Receiver buffer
#define RX_BUFFER_SIZE 24
char rx_buffer[RX_BUFFER_SIZE];

#if RX_BUFFER_SIZE <= 256
unsigned char rx_wr_index,rx_rd_index,rx_counter;
#else
unsigned int rx_wr_index,rx_rd_index,rx_counter;
#endif

// This flag is set on USART Receiver buffer overflow
bit rx_buffer_overflow;

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
char status,data;
status=UCSRA;
data=UDR;
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
   {
   rx_buffer[rx_wr_index++]=data;
#if RX_BUFFER_SIZE == 256
   // special case for receiver buffer size=256
   if (++rx_counter == 0) rx_buffer_overflow=1;
#else
   if (rx_wr_index == RX_BUFFER_SIZE) rx_wr_index=0;
   if (++rx_counter == RX_BUFFER_SIZE)
      {
      rx_counter=0;
      rx_buffer_overflow=1;
      }
#endif
   }
}

#ifndef _DEBUG_TERMINAL_IO_
// Get a character from the USART Receiver buffer
#define _ALTERNATE_GETCHAR_
#pragma used+
char getchar(void)
{
char data;
while (rx_counter==0);
data=rx_buffer[rx_rd_index++];
#if RX_BUFFER_SIZE != 256
if (rx_rd_index == RX_BUFFER_SIZE) rx_rd_index=0;
#endif
#asm("cli")
--rx_counter;
#asm("sei")
return data;
}
#pragma used-
#endif


unsigned char read_adc(unsigned char adc_input)
{
ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
delay_us(10);
ADCSRA|=0x40;
while ((ADCSRA & 0x10)==0);
ADCSRA|=0x10;
return ADCH;
}


void main(void){

unsigned char _temp;
unsigned char _buf_1[17],_r_buf[21];
unsigned char i;

DDRD=(1<<6)|(1<<7);

ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x86;

UCSRA=0x00;
UCSRB=0x98;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x33;



lcd_init(16);
//lcd_clear();
//lcd_putsf("Milad Jahandideh");
//delay_ms(3000);
lcd_clear();
lcd_putsf("Connect USB and ");
delay_ms(1);
lcd_gotoxy(0,1);
lcd_putsf(" Run PC Program ");



RELAY_1=1;
RELAY_2=1;
delay_ms(100);
RELAY_1=0;
RELAY_2=0;

#asm("sei")
while(1){
_temp=read_adc(0);
_temp*=2;
sprintf(_buf_1,"T=%d%cC R1=%d R2=%d",_temp,223,RELAY_1,RELAY_2);

printf("%d",_temp);   //send Temp for PC


  if(getchar()=='R' & getchar()=='L' ) //  RL1 1 HelloE2ND
  {    
    if(getchar()=='1')
    RELAY_1=1;        
    else 
    RELAY_1=0; 
    
    if(getchar()=='1')
    RELAY_2=1;        
    else 
    RELAY_2=0;
    
                 
    
    for(i=0;i<21;i++){
    _r_buf[i]=getchar();                                                       
      if(_r_buf[i-3]=='E' & _r_buf[i-2]=='2' & _r_buf[i-1]=='N' & _r_buf[i]=='D'){ 
      _r_buf[i-3]=0;   
      goto _down;
       }
    }   
    _down:
 
     
    
   
  }  
            
   lcd_clear();
   lcd_puts(_buf_1);
   delay_us(100);
   lcd_gotoxy(0,1);
   lcd_puts(_r_buf);
   delay_ms(150);
}


}